"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const p_spawn_1 = require("p-spawn");
const block_1 = require("./block");
const realm_1 = require("./realm");
const utils_1 = require("./utils");
const hook_1 = require("./hook");
const session_1 = require("./session");
/**
 * Push the local docker image for the service(s) to the remote (tag and push)
 * Assumptions:
 *   - Assume the service names match the docker image convention as is on a local docker repo (localhost:5000/...)
 *   - Today assume remote is gcloud
 * @param realm
 * @param serviceNames
 */
async function push(realm, serviceNames) {
    let names;
    if (serviceNames) {
        names = utils_1.asNames(serviceNames);
    }
    else {
        const dockerBlocks = await block_1.loadDockerBlocks();
        names = Object.keys(dockerBlocks);
    }
    if (realm.registry == null) {
        throw new Error(`Realm '${realm.name}' does not have registry, cannot push docker image.`);
    }
    //// Call prep hook the docker push
    await hook_1.callHook(realm, "dpush_prep", names);
    //// For each service, push the docker image
    for (let serviceName of names) {
        let sourceImage = realm_1.getLocalImageName(realm, serviceName);
        let remoteImage = realm_1.getRemoteImageName(realm, serviceName);
        console.log(`----- Pushing service ${serviceName} : ${remoteImage}`);
        // we make sure the tag exist
        await p_spawn_1.spawn('docker', ['tag', sourceImage, remoteImage]);
        // This assume gcloud has been setup on the local machine
        // Note: gcloud requires `gcloud auth configure-docker` and just use `docker push...
        try {
            await p_spawn_1.spawn('docker', ['push', remoteImage]);
        }
        catch (ex) {
            // Note: Allow the hook to eventually recorver. 
            const passed = await hook_1.callHook(realm, 'dpush_image_ex', ex, remoteImage);
            // If the dpush_image_ex return true, means it was recovered.
            // Otherwise there might not have been any hook or could not recover. (note that the hook could throw ex as well)
            if (passed !== true) {
                throw ex;
            }
        }
        console.log(`----- /DONE pushing ${serviceName} : ${remoteImage} ------\n`);
    }
}
exports.push = push;
async function buildDockerImage(realm, block) {
    const dir = block.dir;
    const imageName = realm_1.getLocalImageName(realm, block.name);
    const start = utils_1.now();
    console.log(`============ BUILDING docker image: ${imageName}\n`);
    // check if this block has a dbuildDependencies
    const dbuildDeps = utils_1.asNames(block.dbuildDependencies);
    for (let dep of dbuildDeps) {
        await block_1.buildBlock(dep);
    }
    // make sure we build the service first
    await block_1.buildBlock(block.name);
    // build the image
    console.log(`------ docker build ... ${imageName}`);
    await p_spawn_1.spawn('docker', ['build', '--rm', '-t', imageName, '.'], { cwd: dir });
    console.log(`------ DONE - docker build ... ${imageName}`);
    console.log();
    // it can be use by future k8s after push
    let status = 'DONE';
    console.log(`------ docker push ... ${imageName}`);
    try {
        if (!session_1.hasSessionState('NO_LOCAL_REGISTRY')) {
            await p_spawn_1.spawn('docker', ['push', imageName]);
        }
    }
    catch (ex) {
        session_1.addSessionState('NO_LOCAL_REGISTRY');
    }
    if (session_1.hasSessionState('NO_LOCAL_REGISTRY')) {
        console.log('INFO - No Local Registry (http://localhost:5000/) - Skipping push to local registry');
        status = "SKIPPING";
    }
    console.log(`------ ${status} - docker push ... ${imageName}`);
    utils_1.printLog(`\n============ DONE - BUILDING docker image: ${imageName}`, null, start);
}
exports.buildDockerImage = buildDockerImage;
